'===$Workfile: dc.vbs $================================================
'dc implements a recursive directory comparison based on fc /b.
'Version    :   $Header: /Bundesarchiv/tocmag/dc/dc.vbs 3     12.07.07 17:26 Hartwig $
'               $NoKeywords: $
'Application:   TOCMAG
'Platform   :   VBScript (http://www.microsoft.com/technet/scriptcenter/guide/
'               and http://msdn.microsoft.com/library/default.asp?url=/library/en-us/script56/html/ddfa5183-d458-41bc-a489-070296ced968.asp)
'Description:   The given directories are compared using fc.
'Prerequisites: fc (handles large files).
'               Make cscript the default host with the command
'               cscript //H:cscript or use cscript.exe explicitly when
'               calling this script.
'-----------------------------------------------------------------------
'Developed  : Hartwig Thomas, Enter AG, Zurich, 2006
'Copyright  2006 Enter AG, Zurich
'  This program is free software; you can redistribute it and/or modify
'  it under the terms of version 3 of the GNU General Public License as
'  published by the Free Software Foundation.
'
'  This program is distributed in the hope that it will be useful,
'  but WITHOUT ANY WARRANTY; without even the implied warranty of
'  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'  GNU General Public License for more details.
'
'  You should have received a copy of the GNU General Public License
'  along with this program.  If not, see <http://www.gnu.org/licenses/>.
'
'  If you have a need for licensing this program without some of the 
'  restrictions specified in the GNU General Public License, it is 
'  possible to negotiate a different license with the copyright holder.
'=======================================================================
Option Explicit

' trace switch can be turned on with /debug on command-line
Dim g_bDebug 
g_bDebug = False
Dim wsn
Set wsn = WScript.Arguments.Named
If wsn.Exists("debug") Then g_bDebug = True

' global objects
Dim g_fso
Set g_fso = WScript.CreateObject("Scripting.FileSystemObject")
Dim g_sh
Set g_sh = WScript.CreateObject("WScript.Shell")

' constants for return values
Const iRETURN_OK = 0
Const iRETURN_WARNING = 4
Const iRETURN_ERROR = 8

' global constant to prevent capturing too large stdout listings
Const iMAX_OUTPUT = 8192
' block size on disk for estimating used space per file
Const iBLOCK_SIZE = 8192
' error marker in stderr
Const sERROR_MARKER = "===ERROR===>"

'=======================================================================
' StdOut and StdErr
'=======================================================================
'-----------------------------------------------------------------------
' out writes sMessage to StdOut
'-----------------------------------------------------------------------
Sub out(sMessage)
  Call WScript.StdOut.Write(sMessage)
End Sub ' out

'-----------------------------------------------------------------------
' outln writes sMessage & vbCrLf to StdOut
'-----------------------------------------------------------------------
Sub outln(sMessage)
  Call WScript.StdOut.WriteLine(sMessage)
End Sub ' outln

'-----------------------------------------------------------------------
' err writes sMessage to StdErr
'-----------------------------------------------------------------------
Sub err(sMessage)
  Call WScript.StdErr.Write(FormatDateTime(Now()) & " " & sERROR_MARKER & " " & sMessage)
End Sub ' errln

'-----------------------------------------------------------------------
' errln writes sMessage & vbCrLf to StdErr
'-----------------------------------------------------------------------
Sub errln(sMessage)
  Call WScript.StdErr.WriteLine(FormatDateTime(Now()) & " " & sERROR_MARKER & " " & sMessage)
End Sub ' errln

'=======================================================================
' Tracing for debugging
'=======================================================================
' trace indent
Dim g_iIndent
g_iIndent = 0
'-----------------------------------------------------------------------
' trace displays the given message, if g_bDebug is True
'-----------------------------------------------------------------------
Function trace(sMessage)
  On Error Resume Next
  If g_bDebug Then 
    Call outln(FormatDateTime(Now()) & " " & String(g_iIndent," ") & sMessage)
    trace = True
  Else
    trace = False
  End If
End Function ' trace

'-----------------------------------------------------------------------
' traceEntry traces function/procedure entry
'-----------------------------------------------------------------------
Function traceEntry(sProcedure,sParameters)
  On Error Resume Next
  Dim sMessage
  sMessage = ">>> " & sProcedure
  If Len(sParameters) > 0 Then sMessage = sMessage & "(" & sParameters & ")"
  traceEntry = trace(sMessage)
  g_iIndent = g_iIndent + 2
End Function ' traceEntry

'-----------------------------------------------------------------------
' traceExit traces function/procedure exit
'-----------------------------------------------------------------------
Function traceExit(sProcedure,sReturn)
  On Error Resume Next
  Dim sMessage
  g_iIndent = g_iIndent - 2
  sMessage = "<<< " & sProcedure
  If Len(sReturn) > 0 Then sMessage = sMessage & "(" & sReturn & ")"
  traceExit = trace(sMessage)
End Function ' traceExit

'=======================================================================
' Utilities
'=======================================================================
'-----------------------------------------------------------------------
' runCommand uses g_sh.Exec, and Wait for running a command
' and returns the exit code. If requested, the contents of StdOut and StdErr
' are piped through.
'-----------------------------------------------------------------------
Function runCommand(sCommand,bOutput,bError,sOutput,sError)
  On Error Resume Next
  traceEntry "runCommand", sCommand & "," & bOutput & "," & bError
  Dim sOut
  Dim sErr
  Dim wse
  Dim iMaxOutput
  Dim iMaxError
  Dim sChar
  ' initialize sOutput
  iMaxOutput = Len(sOutput)
  If iMaxOutput > 0 Then 
    sOutput = ""
  End If
  ' initialize sError
  iMaxError = Len(sError)
  If Not iMaxError > 0 Then
    sError = ""
  End If
  ' execute the command
  Set wse = g_sh.Exec(sCommand)
  ' do a not terribly busy loop, waiting for completion
  While wse.Status = 0
    WScript.Sleep 100
    ' if there is output then read it
    If Not wse.StdOut.AtEndOfStream Then 
      sChar = wse.StdOut.Read(1)
      While Len(sChar) > 0
        ' if output is to be piped then write it
        If bOutput Then Call out(sChar)
        ' if output is to be captured then capture it
        If iMaxOutput > 0 Then
          If Len(sOutput) < iMaxOutput Then
            sOutput = sOutput & sChar
          Else
            sOutput = Right(sOutput,iMaxOutput - 1) & sChar
          End If
        End If
        sChar = wse.StdOut.Read(1)
      Wend
    End If
    ' if there is error output then read it
    If Not wse.StdErr.AtEndOfStream Then 
      sChar = wse.StdOut.Read(1)
      While Len(sChar) > 0
        ' if error output is to be piped then write it
        If bError Then Call err(sChar)
        ' if error output is to be captured then capture it
        if iMaxError > 0 Then
          If Len(sError) < iMaxError Then
            sError = sError & sChar
          Else
            sError = Right(sError,iMaxError - 1) & sChar
          End If
        End If
        sChar = wse.StdOut.Read(1)
      Wend
    End If
  Wend
  ' return exit code
  Dim iReturn
  iReturn = wse.ExitCode
  traceExit "runCommand", iReturn
  runCommand = iReturn
End Function ' runCommand

'=======================================================================
' Calling syntax and command-line arguments
'=======================================================================
'-----------------------------------------------------------------------
' echoHelp
'   displays calling syntax
'-----------------------------------------------------------------------
Sub echoHelp(sDir1, sDir2)
  On Error Resume Next
  traceEntry "echoHelp", sDir1 & "," & sDir2
  Call outln("dc compares the given directories using fc /b")
  Call outln("")
  Call outln("Calling syntax:")
  Call outln("  cscript dc.vbs /h | <dir1> <dir2>")
  Call outln("Options:")
  Call outln(" /h      if present, this help text is displayed")
  Call outln(" <dir1>  first directory to be compared")
  Call outln(" <dir2>  second directory to be compared")
  Call outln("")
  traceExit "echoHelp", ""
End Sub ' echoHelp

'-----------------------------------------------------------------------
' getParameters
'   analyzes the command line and returns the parameter values
'-----------------------------------------------------------------------
Function getParameters(sDir1,sDir2)
  On Error Resume Next
  traceEntry "getParameters", sDir1 & ", " & sDir2
  Dim bReturn
  bReturn = False
  ' named arguments
  Dim wsn
  Set wsn = WScript.Arguments.Named
  ' unnamed arguments
  Dim wsu
  Set wsu = WScript.Arguments.Unnamed
  If wsn.Exists("h")Then
  ElseIf wsn.Exists("?") Then
  ElseIf wsn.Exists("help") Then
  Else
    sDir1 = wsu.Item(0)
    sDir2 = wsu.Item(1)
    bReturn = True
  End If
  traceExit "getParameters", bReturn
  getParameters = bReturn
End Function ' getParameters

'-----------------------------------------------------------------------
' echoParameters
'   echos the parameters used for the skript
'-----------------------------------------------------------------------
Sub echoParameters(sDir1, sDir2)
  On Error Resume Next
  traceEntry "echoParameters", sDir1 & "," & sDir2
  Call outln("  Parameter values:")
  Call outln("    dir1: """ & sDir1 & """")
  Call outln("    dir2: """ & sDir2 & """")
  Call outln("")
  traceExit "echoParameters", ""
End Sub ' echoParameters

'=======================================================================
' Prerequisites
'=======================================================================
'-----------------------------------------------------------------------
' checkPrerequisites
'-----------------------------------------------------------------------
Function checkPrerequisites(sDir1,sDir2)
  On Error Resume Next
  traceEntry "checkPrerequisites", sDir1 & "," & sDir2
  Dim bReturn
  If Not g_fso.FolderExists(sDir1) Then
    Call errln("Directory " & sDir1 & " does not exist!")
    bReturn = False
  ElseIf Not g_fso.FolderExists(sDir2) Then
    Call errln("Directory " & sDir2 & " does not exist!")
    bReturn = False
  Else
    bReturn = True
  End If
  traceExit "checkPrerequisites", bReturn
  checkPrerequisites = bReturn
End Function ' checkPrerequisites

'=======================================================================
' compareDirectories
'=======================================================================
'-----------------------------------------------------------------------
' compareFiles
'-----------------------------------------------------------------------
Function compareFiles(file1, file2)
  On Error Resume Next
  traceEntry "compareFiles", file1.Path & "," & file2.Path
  Dim iReturn
  Dim sCommand
  Dim iAttributes
  ' issue a spurious attrib command because of a strange fc error
  iAttributes = file1.attributes
  If iAttributes = 0 Then
    file1.attributes = iARCHIVE
  End If
  iReturn = iRETURN_ERROR
  sCommand = "fc /b """ & file1.Path & """ """ & file2.Path & """"
  trace sCommand
  iReturn = runCommand(sCommand,g_bDebug,True,"","")
  If iAttributes = 0 Then
    file1.attributes = 0
  End If
  traceExit "compareFiles", iReturn
  compareFiles = iReturn
End Function ' compareFiles

'-----------------------------------------------------------------------
' compareFolders
'-----------------------------------------------------------------------
Function compareFolders(folderFirst, folderSecond)
  On Error Resume Next
  traceEntry "compareFolders", folderFirst.Path & ", " & folderSecond.Path
  Dim iReturn
  Dim iCount1
  Dim iCount2
  Dim files1
  Dim files2
  Dim file1
  Dim file2
  Dim sFile2
  Dim folders1
  Dim folders2
  Dim folder1
  Dim folder2
  Dim sFolder2
  iReturn = iRETURN_OK
  trace "Files ..."
  If iReturn = iRETURN_OK Then
    Set files1 = folderFirst.Files
    iCount1 = files1.Count
    Set files2 = folderSecond.Files
    iCount2 = files2.Count
    trace "iCount1 " & iCount1
    trace "iCount2 " & iCount2
    If iCount1 = iCount2 Then
      For Each file1 in files1
        trace "file1 " & file1.Path
        If iReturn = iRETURN_OK Then
          sFile2 = folderSecond.Path & "\" & file1.Name
          trace "file2 " & sFile2
          If g_fso.FIleExists(sFile2) Then
            Set file2 = g_fso.GetFile(sFile2)
            iReturn = compareFiles(file1,file2)
          Else
            outln("File """ & file1.Name & """ does not exist in """ & folderSecond.Path & "!")
            iReturn = iRETURN_ERROR
          End If
        End If
      Next
    Else
      outln("folders """ & folderFirst.Path & """ and """ & folderSecond.Path & """ contain an different number of files!")
      iReturn = iRETURN_ERROR
    End If
  End If
  trace "Subfolders ..."
  If iReturn = iRETURN_OK Then
    Set folders1 = folderFirst.Subfolders
    iCount1 = folders1.Count
    Set folders2 = folderSecond.Subfolders
    iCount2 = folders2.Count
    trace "iCount1 " & iCount1
    trace "iCount2 " & iCount2
    If iCount1 = iCount2 Then
      For Each folder1 in folders1
        trace "folder1 " & folder1.Path
        If iReturn = iRETURN_OK Then
          sFolder2 = folderSecond.Path & "\" & folder1.Name
          trace "folder2 " & sFolder2
          If g_fso.FolderExists(sFolder2) Then
            Set folder2 = g_fso.GetFolder(sFolder2)
            iReturn = compareFolders(folder1,folder2)
          Else
            outln("Folder """ & folder1.Name & """ does not exist in """ & folderSecond.Path & "!")
            iReturn = iRETURN_ERROR
          End If
        End If
      Next
    Else
      outln("folders """ & folderFirst.Path & """ and """ & folderSecond.Path & """ contain an different number of subfolders!")
      iReturn = iRETURN_ERROR
    End If
  End If  
  traceExit "compareFolders", iReturn
  compareFolder = iReturn
End Function ' compareFolders

'=======================================================================
' Main
'=======================================================================
'-----------------------------------------------------------------------
' main
'-----------------------------------------------------------------------
Sub Main()
  On Error Resume Next
  traceEntry "main", ""
  ' return code
  Dim iReturn
  ' parameters
  Dim sDir1
  Dim sDir2
  ' Initialize return code
  iReturn = iRETURN_OK
  ' Default values for parameters
  sDir1 = ""
  sDir2 = ""
  If getParameters(sDir1,sDir2) Then
    ' banner with time stamp
    outln(FormatDateTime(Now()) & " dc ; Copyright 2006, 2008 Enter AG, Zurich")
    Call echoParameters(sDir1,sDir2)
    If checkPrerequisites(sDir1,sDir2) Then
      Dim folder1
      Dim folder2
      Set folder1 = g_fso.GetFolder(sDir1)
      Set folder2 = g_fso.GetFolder(sDir2)
      iReturn = compareFolders(folder1,folder2)
    End If
    If iReturn = iRETURN_OK Then
      outln(FormatDateTime(Now()) & " directories """+folder1.Path+""" and """+folder2.Path+""" are equal.")
    Else
      outln(FormatDateTime(Now()) & " directories """+folder1.Path+""" and """+folder2.Path+""" are not equal.")
    End If
  Else
    Call echoHelp(sDir1,sDir2)
    iReturn = iRETURN_WARNING
  End If
  traceExit "main", iReturn
  WScript.quit(iReturn)
End Sub ' Main

'-----------------------------------------------------------------------
Call Main()

'=== End of file =======================================================
